#### Function to merge Table A and biological tables




mergeCatchesAndBiologicalTables <- function(WD , 
                                        ageOrLength ) {
  
  #load tables-----
  catchesTot <- readRDS(file = paste0(WD,"catchTot.rds"))
  discardsStruc <- readRDS(file = paste0(WD, "discardsStruc", ageOrLength, ".rds"))
  landingsStruc <- readRDS(file = paste0(WD, "landingsStruc", ageOrLength, ".rds"))
  
  
  
  ## varaible to define acceptable difference between SOP from biological tables and sum over a given domain in table A
  diffSOP <- 0.1
  ## compute sum land/dis table A--------------
  landingsTotSum <- catchesTot %>%
    group_by(country, year, species, domain_landings, nep_sub_region) %>% 
    summarise(totalLandA = sum( total_live_weight_landed , na.rm=T), 
              conf = length(which(confidential %in% c("A", "V"))))
  discardsTotSum <- catchesTot %>%
    group_by(country, year, species, domain_discards, nep_sub_region) %>% 
    summarise(totalDisA = sum( tot_discards_tonnes , na.rm=T), 
              conf = length(which(confidential %in% c("A", "V"))))
  

  if(ageOrLength == "length") {
    ###@length------
        #### compute SOP in tables C-F to compare with Table A and extract/plot mean weights when abs(weight/SOP) > 10%--------------
        ###discards-----
        discardsStrucSOP <- discardsStruc %>% 
          group_by(country, year, species, domain_discards, nep_sub_region)  %>%
          summarise(SOPDis = sum(no_length_discards * mean_weight_discards, na.rm=T), 
                    tot_discards_tonnes = unique(tot_discards_tonnes)) %>%
          filter(SOPDis > 0)
        
        discardsStrucSOP <- full_join(discardsTotSum, discardsStrucSOP) %>%
          mutate(diff = SOPDis / tot_discards_tonnes ) %>%  # difference between weight in biological tables and SOP in biological table, remove units problems
          filter(domain_discards != "NK" & SOPDis>0)
        
        domainsDiscardToKeep <- discardsStrucSOP %>%
          filter( (diff > (1-diffSOP) & diff < (1+diffSOP)) & totalDisA - tot_discards_tonnes <1) %>% # difference between weight in TableA and weight in biological table, remove rows were weight in table A higher as should be lower because some rows in Table A are marked as confidential
          dplyr::select(country, year, domain_discards, species, nep_sub_region) %>% 
          distinct() %>% 
          mutate(toKeep = "T")
        
        
        ###landings-----
        landingsStrucSOP <- landingsStruc %>% 
          group_by(country, year, species, domain_landings, nep_sub_region)  %>%
          summarise(SOPLand = sum(no_length_landings * mean_weight_landings, na.rm=T),
                    total_live_weight_landed = unique(total_live_weight_landed)) %>%
          filter(SOPLand > 0)
        
        landingsStrucSOP <- full_join(landingsTotSum, landingsStrucSOP) %>%
          mutate(diff = SOPLand / total_live_weight_landed ) %>% # difference between weight in biological tables and SOP in biological table, remove units problems
          filter(domain_landings != "NK" & SOPLand>0)
        
        domainsLandingToKeep <- landingsStrucSOP %>%
          filter( (diff > (1-diffSOP) & diff < (1+diffSOP))  & totalLandA - total_live_weight_landed<1) %>% 
          dplyr::select(country, year, domain_landings, species, nep_sub_region) %>% 
          distinct() %>% 
          mutate(toKeep = "T")
        
        ### merge ------  
        ## landings
        ### compute SOP and keep all rows of biological tables
        landingsStruc <- landingsStruc %>%
          group_by(country, year, domain_landings, nep_sub_region, species) %>%
          mutate(SOP = sum(no_length_landings * mean_weight_landings, na.rm=T)) %>%
          ungroup() %>% 
          rename(total_live_weight_landed_biologicalTable = total_live_weight_landed)
        
        
        tableA_Land <- full_join(catchesTot, 
                                 landingsStruc %>% 
                                    filter( paste(country, year, domain_landings, species) %in%
                                              paste(domainsLandingToKeep$country,  
                                                    domainsLandingToKeep$year, 
                                                    domainsLandingToKeep$domain_landings, 
                                                    domainsLandingToKeep$species)
                                    )) 
        tableA_Land <- tableA_Land %>%
          mutate(no_length_landings = no_length_landings * total_live_weight_landed/SOP) %>%
          dplyr::select(-SOP)
        
        tableA_Land   %>% filter(!is.na(length))%>%
          summarise(sum(unique(total_live_weight_landed), na.rm=T))
        tableA_Land %>%
          summarise(sum(mean_weight_landings * no_length_landings, na.rm=T))
        
        ## discards
        discardsStruc <- discardsStruc %>%
          group_by(country, year, domain_discards, nep_sub_region, species) %>%
          mutate(SOP = sum(no_length_discards * mean_weight_discards, na.rm=T)) %>%
          ungroup() %>% 
          rename(tot_discards_tonnes_biologicalTable = tot_discards_tonnes) %>%
          dplyr::select(-total_live_weight_landed)
        
        
        tableA_Disc <- full_join(catchesTot, 
                                discardsStruc %>% 
                                  rename(no_length_measurements_discards = no_length_measurements,
                                         length_unit_discards = length_unit,
                                         min_lengtht_discards = min_length,
                                         max_lengtht_discards = max_length,
                                         weight_unit_discards = weight_unit, 
                                         total_sampled_trips_discards = total_sampled_trips) %>% 
                                
                                
                                   filter( paste(country, year, domain_discards, species) %in%
                                             paste(domainsDiscardToKeep$country,  
                                                   domainsDiscardToKeep$year, 
                                                   domainsDiscardToKeep$domain_discards, 
                                                   domainsDiscardToKeep$species)
                                   ))
        
        tableA_Disc <- tableA_Disc %>%
          mutate(no_length_discards = no_length_discards * tot_discards_tonnes/SOP) %>%
          dplyr::select(-SOP) 
        
        
        
        tableA_AtLength <- full_join(tableA_Land, tableA_Disc)
        
        tableA_AtLength <- tableA_AtLength %>%
          dplyr::select(colnames(catchesTot),
                        length, length_unit, no_length_landings, mean_weight_landings, weight_unit,
                        no_length_discards, mean_weight_discards)
        
      #  tableA_AtLength  %>% filter(!is.na(mean_weight_discards)) %>%
      #    summarise(sum(unique(tot_discards_tonnes), na.rm=T))
      #  tableA_AtLength %>%
      #    summarise(sum(mean_weight_discards * no_length_discards, na.rm=T))
        
        saveRDS(tableA_AtLength, file = paste0(WD,"tableA_AtLength.rds"))


  } else {
    ###@age------
    #### compute SOP in tables C-F to compare with Table A and extract/plot mean weights when abs(weight/SOP) > 10%--------------
    ###discards-----
    discardsStrucSOP <- discardsStruc %>% 
      group_by(country, year, species, domain_discards, nep_sub_region)  %>%
      summarise(SOPDis = sum(no_age_discards * mean_weight_discards, na.rm=T), 
                tot_discards_tonnes = unique(tot_discards_tonnes)) %>%
      filter(SOPDis > 0)
    
    discardsStrucSOP <- full_join(discardsTotSum, discardsStrucSOP) %>%
      mutate(diff = SOPDis / tot_discards_tonnes ) %>%  # difference between weight in biological tables and SOP in biological table, remove units problems
      filter(domain_discards != "NK" & SOPDis>0)
    
    domainsDiscardToKeep <- discardsStrucSOP %>%
      filter( (diff > (1-diffSOP) & diff < (1+diffSOP)) & totalDisA - tot_discards_tonnes <1) %>% # difference between weight in TableA and weight in biological table, remove rows were weight in table A higher as should be lower because some rows in Table A are marked as confidential
      dplyr::select(country, year, domain_discards, species, nep_sub_region) %>% 
      distinct() %>% 
      mutate(toKeep = "T")
    
    
    ###landings-----
    landingsStrucSOP <- landingsStruc %>% 
      group_by(country, year, species, domain_landings, nep_sub_region)  %>%
      summarise(SOPLand = sum(no_age_landings * mean_weight_landings, na.rm=T),
                total_live_weight_landed = unique(total_live_weight_landed)) %>%
      filter(SOPLand > 0)
    
    landingsStrucSOP <- full_join(landingsTotSum, landingsStrucSOP) %>%
      mutate(diff = SOPLand / total_live_weight_landed ) %>% # difference between weight in biological tables and SOP in biological table, remove units problems
      filter(domain_landings != "NK" & SOPLand>0)
    
    domainsLandingToKeep <- landingsStrucSOP %>%
      filter( (diff > (1-diffSOP) & diff < (1+diffSOP))  & totalLandA - total_live_weight_landed<1) %>% 
      dplyr::select(country, year, domain_landings, species, nep_sub_region) %>% 
      distinct() %>% 
      mutate(toKeep = "T")
    
    ### merge ------  
    ## landings
    ### compute SOP and keep all rows of biological tables
    landingsStruc <- landingsStruc %>%
      group_by(country, year, domain_landings, nep_sub_region, species) %>%
      mutate(SOP = sum(no_age_landings * mean_weight_landings, na.rm=T)) %>%
      ungroup() %>% 
      rename(total_live_weight_landed_biologicalTable = total_live_weight_landed)
    
    
    tableA_Land <- full_join(catchesTot, 
                             landingsStruc %>% 
                               filter( paste(country, year, domain_landings, species) %in%
                                         paste(domainsLandingToKeep$country,  
                                               domainsLandingToKeep$year, 
                                               domainsLandingToKeep$domain_landings, 
                                               domainsLandingToKeep$species)
                               )) 
    tableA_Land <- tableA_Land %>%
      mutate(no_age_landings = no_age_landings * total_live_weight_landed/SOP) %>%
      dplyr::select(-SOP)
    
    tableA_Land   %>% filter(!is.na(age))%>%
      summarise(sum(unique(total_live_weight_landed), na.rm=T))
    tableA_Land %>%
      summarise(sum(mean_weight_landings * no_age_landings, na.rm=T))
    
    ## discards
    discardsStruc <- discardsStruc %>%
      group_by(country, year, domain_discards, nep_sub_region, species) %>%
      mutate(SOP = sum(no_age_discards * mean_weight_discards, na.rm=T)) %>%
      ungroup() %>% 
      rename(tot_discards_tonnes_biologicalTable = tot_discards_tonnes) %>%
      dplyr::select(-total_live_weight_landed)
    
    
       
    tableA_Disc <- full_join(catchesTot, 
                                 discardsStruc %>% 
                                   rename(no_age_measurements_discards = no_age_measurements,
                                          age_measurements_prop_discards = age_measurements_prop,
                                          length_unit_discards = length_unit,
                                          min_age_discards = min_age,
                                          max_age_discards = max_age,
                                          weight_unit_discards = weight_unit, 
                                          total_sampled_trips_discards = total_sampled_trips,
                                          mean_length_discards = mean_length) %>% 
                                   
                                   
                                   filter( paste(country, year, domain_discards, species) %in%
                                             paste(domainsDiscardToKeep$country,  
                                                   domainsDiscardToKeep$year, 
                                                   domainsDiscardToKeep$domain_discards, 
                                                   domainsDiscardToKeep$species)
                                   )) 
    
    
    tableA_Disc <- tableA_Disc %>%
      mutate(no_age_discards = no_age_discards * tot_discards_tonnes/SOP) %>%
      dplyr::select(-SOP) 
    
    tableA_Disc   %>% filter(!is.na(age))%>%
      summarise(sum(unique(total_live_weight_landed), na.rm=T))
    tableA_Disc %>%
      summarise(sum(mean_weight_discards * no_age_discards, na.rm=T))
    
    tableA_AtAge <- full_join(tableA_Land, tableA_Disc)
    
    
    tableA_AtAge <- tableA_AtAge %>%
      dplyr::select(colnames(catchesTot),
                    age, no_age_landings, mean_weight_landings, weight_unit,mean_length, length_unit,
                    no_age_discards, mean_weight_discards, mean_length_discards)
    
   # tableA_AtAge  %>% filter(!is.na(mean_weight_discards)) %>%
   #    summarise(sum(unique(tot_discards_tonnes), na.rm=T))
   # tableA_AtAge %>%
   #   summarise(sum(mean_weight_discards * no_age_discards, na.rm=T))
    
    saveRDS(tableA_AtAge, file = paste0(WD,"tableA_AtAge.rds"))
    
    
    
    
    
  }
  
  
  
  
  
  
}


