#################################################################################################################################################################
#################################################################################################################################################################
#######################                                                                                                                         #################
####################### Script to merge catch by country table and biological (age/length structure tables)                                     #################
####################### ! When merging these tables, it should be kept in mind that Domains are defined to match the sampling program           #################
####################### designs and ensure the raising procedures are statistically sound. Raising procedures concern discards estimates        #################
####################### but also landings/discards length/age structures. One domain then corresponds to one line in biological tables (C-F)    #################
####################### but potentially multiple gear/mesh size/quarters in table A depending on the strata aggregation of the sampling design. #################
#######################                                                                                                                         #################
####################### Merging catch table and biological tables allow defining a length/age structure for each line of the catch table        #################
####################### but do not follow sampling design anymore. The big hypothesis done is that the length/age and discard ratio are         #################
####################### homogeneous inside a domain which might not be the case                                                                 #################
#######################                                                                                                                         #################  
#######################  merging is made on :                                                                                                   #################
####################### COUNTRY_CODE, YEAR, DOMAIN_LANDINGS, SPECIES, NEP_SUB_REGION for landings and                                           #################
####################### COUNTRY_CODE, YEAR, DOMAIN_DISCARDS, SPECIES, NEP_SUB_REGION for discards.                                              #################
#######################                                                                                                                         #################    
####################### the script produces intermediate tables and final tables are tableA_AtAge.rds if                                        #################
####################### age structured data are asked or tableA_AtLength.rds                                                                    #################  
####################### These tables are saved in the folder defined by WDData                                                                  #################  
#######################                                                                                                                         #################  
#######################   ! can take some time to run                                                                                           #################  
#################################################################################################################################################################
#################################################################################################################################################################

### library used in the scripts
library(dplyr)

### clear workspace
rm(list=ls())
gc(reset = T)


### 
WDData <-  "./FDI 2023/dissemination 2023/Final/merge table A and biological tables/"  ### working directory where the data downloaded from the dissemination website have been unzipped TO BE CHANGED BY THE USER!!!
WDScript <-  "./FDI 2023/dissemination 2023/script dissemination biological table updated/"  ### working directory where the scripts downloaded from the dissemination website have been unzipped TO BE CHANGED BY THE USER!!!
AgeOrLength = "length"                        ### can either be "length" if the interest is length structures or "age"
ListYears = c(2013:2021)                   ### time series of catches by country available in the zip file. Time series available after the EWG 22-10


source(file = paste0(WDScript, "load_format_function.r"))   
source(file = paste0(WDScript, "mergeFunction.r"))

loadAndFormatSTECF_FDI_DATA( WD = WDData, ageOrLength = AgeOrLength, listYears = ListYears)
mergeCatchesAndBiologicalTables( WD = WDData, ageOrLength = AgeOrLength)
